package ch.dlmw.swisssignchallenge.controllers;

import ch.dlmw.swisssignchallenge.entities.SigningRequestDocument;
import ch.dlmw.swisssignchallenge.services.SigningRequestDocumentService;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.fge.jsonpatch.JsonPatch;
import com.github.fge.jsonpatch.JsonPatchException;
import lombok.AllArgsConstructor;
import org.openapitools.api.SigningRequestDocumentApi;
import org.openapitools.model.PatchOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.util.List;

@RestController
@AllArgsConstructor
public class SigningRequestDocumentController implements SigningRequestDocumentApi {
    private final SigningRequestDocumentService signingRequestDocumentService;
    private final ObjectMapper objectMapper;

    @Override
    public ResponseEntity<Void> patchSigningRequestDocument(String id, List<PatchOperation> patchOperations) {
        // Fetch the existing document
        var document = signingRequestDocumentService.getSigningRequestDocument(id);

        try {
            ObjectMapper mapper = new ObjectMapper();
            // Serialize patchOperations to JsonNode
            JsonNode patchNode = mapper.valueToTree(patchOperations);

            // Create JsonPatch from JsonNode
            JsonPatch jsonPatch = JsonPatch.fromJson(patchNode);

            // Apply patch to the document's JsonNode
            JsonNode patchedNode = jsonPatch.apply(mapper.convertValue(document, JsonNode.class));

            // Convert the patched JsonNode back to SigningRequestDocument
            SigningRequestDocument patchedDocument = mapper.treeToValue(patchedNode, SigningRequestDocument.class);

            // Now, update the document in the service layer
            signingRequestDocumentService.updateSigningRequestDocument(patchedDocument);

            return ResponseEntity.noContent().build();
        } catch (IOException | JsonPatchException e) {
            // Handle exceptions appropriately
            throw new RuntimeException("Failed to apply patch", e);
        }
    }

}
